@@ -4,7 +4,6 @@
 #
 .onLoad <- function(lib, pkg)
 {
-    library.dynam("RODBC", pkg, lib)
     if(is.null(getOption("dec")))
         options(dec = Sys.localeconv()$decimal_point)
 }
@@ -24,9 +23,8 @@ odbcGetErrMsg <- function(channel)
 {
     if(!odbcValidChannel(channel))
        stop("first argument is not an open RODBC channel")
-    err <- .Call("RODBCGetErrMsg", attr(channel, "handle_ptr"),
-                 PACKAGE = "RODBC")
-    .Call("RODBCClearError", attr(channel, "handle_ptr"), PACKAGE = "RODBC")
+    err <- .Call(C_RODBCGetErrMsg, attr(channel, "handle_ptr"))
+    .Call(C_RODBCClearError, attr(channel, "handle_ptr"))
     return(err)
 }
 
@@ -34,7 +32,7 @@ odbcClearError <- function(channel)
 {
     if(!odbcValidChannel(channel))
        stop("first argument is not an open RODBC channel")
-    .Call("RODBCClearError", attr(channel, "handle_ptr"), PACKAGE = "RODBC")
+    .Call(C_RODBCClearError, attr(channel, "handle_ptr"))
     invisible()
 }
 
@@ -71,23 +69,23 @@ odbcDriverConnect <-
  	stop("Invalid case parameter: nochange | toupper | tolower | common db names")
 	)
    id <- as.integer(1 + runif(1, 0, 1e5))
-   stat <- .Call("RODBCDriverConnect", as.character(connection), id,
-                 as.integer(believeNRows), PACKAGE = "RODBC")
+   stat <- .Call(C_RODBCDriverConnect, as.character(connection), id,
+                 as.integer(believeNRows))
    if(stat < 0) {
        warning("ODBC connection failed")
        return(stat)
    }
    case <- switch(case+1, "nochange", "toupper", "tolower")
-   return(structure(stat, class="RODBC", case=case, id = id,
-                    believeNRows=believeNRows))
+   structure(stat, class="RODBC", case=case, id = id,
+             believeNRows=believeNRows)
 }
 
-odbcQuery <- function(channel, query)
+odbcQuery <- function(channel, query, rows_at_time = 1)
 {
     if(!odbcValidChannel(channel))
        stop("first argument is not an open RODBC channel")
-    .Call("RODBCQuery", attr(channel, "handle_ptr"), as.character(query),
-          PACKAGE = "RODBC")
+    .Call(C_RODBCQuery, attr(channel, "handle_ptr"), as.character(query),
+          as.integer(rows_at_time))
 }
 
 odbcUpdate <-
@@ -100,48 +98,45 @@ odbcUpdate <-
     if(verbose) vflag <- 1
     if(test) vflag <- 2
     ## apply the name mangling that was applied when the table was created
-    cnames<- gsub("[^A-Za-z0-9_]+", "", as.character(colnames(data)))
+    cnames <- mangleColNames(names(data))
     cnames <- switch(attr(channel, "case"),
                      nochange = cnames,
                      toupper = toupper(cnames),
                      tolower = tolower(cnames))
     for(i in 1:ncol(data))
-        if(!is.numeric(data[[i]]))
-           data[[i]] <- as.character(data[[i]])
-    .Call("RODBCUpdate", attr(channel, "handle_ptr"), as.character(query),
+        if(!is.numeric(data[[i]])) data[[i]] <- as.character(data[[i]])
+    .Call(C_RODBCUpdate, attr(channel, "handle_ptr"), as.character(query),
           data, cnames, as.integer(nrow(data)), as.integer(ncol(data)),
-          as.character(params), as.integer(vflag), PACKAGE = "RODBC")
+          as.character(params), as.integer(vflag))
 }
 
 odbcTables <- function(channel)
 {
     if(!odbcValidChannel(channel))
        stop("first argument is not an open RODBC channel")
-    .Call("RODBCTables", attr(channel, "handle_ptr"), PACKAGE = "RODBC")
+    .Call(C_RODBCTables, attr(channel, "handle_ptr"))
 }
 
 odbcColumns <- function(channel, table)
 {
     if(!odbcValidChannel(channel))
        stop("first argument is not an open RODBC channel")
-    .Call("RODBCColumns", attr(channel, "handle_ptr"), as.character(table),
-          PACKAGE = "RODBC")
+    .Call(C_RODBCColumns, attr(channel, "handle_ptr"), as.character(table))
 }
 
 odbcSpecialColumns <- function(channel, table)
 {
     if(!odbcValidChannel(channel))
        stop("first argument is not an open RODBC channel")
-    .Call("RODBCSpecialColumns", attr(channel, "handle_ptr"), as.character(table),
-          PACKAGE = "RODBC")
+    .Call(C_RODBCSpecialColumns, attr(channel, "handle_ptr"),
+          as.character(table))
 }
 
 odbcPrimaryKeys <- function(channel, table)
 {
     if(!odbcValidChannel(channel))
         stop("first argument is not an open RODBC channel")
-    .Call("RODBCPrimaryKeys", attr(channel, "handle_ptr"), as.character(table),
-          PACKAGE = "RODBC")
+    .Call(C_RODBCPrimaryKeys, attr(channel, "handle_ptr"), as.character(table))
 }
 
 ## this does no error checking, but may add an error message
@@ -149,14 +144,14 @@ odbcColData <- function(channel)
 {
     if(!odbcValidChannel(channel))
        stop("first argument is not an open RODBC channel")
-    .Call("RODBCColData", attr(channel, "handle_ptr"), PACKAGE = "RODBC")
+    .Call(C_RODBCColData, attr(channel, "handle_ptr"))
 }
 
 odbcNumCols <- function(channel)
 {
     if(!odbcValidChannel(channel))
        stop("first argument is not an open RODBC channel")
-    .Call("RODBCNumCols", attr(channel, "handle_ptr"), PACKAGE = "RODBC")
+    .Call(C_RODBCNumCols, attr(channel, "handle_ptr"))
 }
 
 close.RODBC <- function(con, ...) odbcClose(con)
@@ -165,7 +160,7 @@ odbcClose <- function(channel)
 {
     if(!odbcValidChannel(channel))
        stop("argument is not an open RODBC channel")
-    res <- .Call("RODBCClose", attr(channel, "handle_ptr"), PACKAGE = "RODBC")
+    res <- .Call(C_RODBCClose, attr(channel, "handle_ptr"))
     if(res > 0) invisible(TRUE) else {
         warning(paste(odbcGetErrMsg(channel), sep="\n"))
         FALSE
@@ -174,7 +169,7 @@ odbcClose <- function(channel)
 
 odbcCloseAll <- function()
 {
-    .Call("RODBCCloseAll", PACKAGE = "RODBC")
+    .Call(C_RODBCCloseAll)
     invisible()
 }
 
@@ -184,8 +179,8 @@ odbcFetchRows <-
 {
     if(!odbcValidChannel(channel))
        stop("first argument is not an open RODBC channel")
-    .Call("RODBCFetchRows", attr(channel, "handle_ptr"), max, buffsize,
-          as.character(nullstring), believeNRows, PACKAGE = "RODBC")
+    .Call(C_RODBCFetchRows, attr(channel, "handle_ptr"), max, buffsize,
+          as.character(nullstring), believeNRows)
 }
 
 odbcCaseFlag <- function (channel)
@@ -199,7 +194,7 @@ odbcGetInfo <- function(channel)
 {
     if(!odbcValidChannel(channel))
        stop("first argument is not an open RODBC channel")
-    res <- .Call("RODBCGetInfo", attr(channel, "handle_ptr"), PACKAGE = "RODBC")
+    res <- .Call(C_RODBCGetInfo, attr(channel, "handle_ptr"))
     names(res) <- c("DBMS_Name", "DBMS_Ver", "Driver_ODBC_Ver",
                     "Data_Source_Name", "Driver_Name", "Driver_Ver",
                     "ODBC_Ver", "Server_Name")
@@ -209,15 +204,14 @@ odbcGetInfo <- function(channel)
 odbcValidChannel <-  function(channel)
 {
     inherits(channel, "RODBC") && is.integer(channel) &&
-    .Call("RODBCcheckchannel", channel, attr(channel, "id"),
-          PACKAGE = "RODBC") > 0
+    .Call(C_RODBCcheckchannel, channel, attr(channel, "id")) > 0
 }
 
 odbcClearResults <-  function(channel)
 {
     if(!odbcValidChannel(channel))
        stop("first argument is not an open RODBC channel")
-    .Call("RODBCclearresults", attr(channel, "handle_ptr"), PACKAGE = "RODBC")
+    .Call(C_RODBCclearresults, attr(channel, "handle_ptr"))
     invisible()
 }
 
@@ -229,3 +223,24 @@ print.RODBC <- function(x, ...)
     cat(case, con, sep="\n  ")
     invisible(x)
 }
+
+odbcSetAutoCommit <- function(channel, autoCommit = TRUE)
+{
+    if(!odbcValidChannel(channel))
+         stop("first argument is not an open RODBC channel")
+    .Call(C_RODBCSetAutoCommit, attr(channel, "handle_ptr"), autoCommit)
+}
+
+odbcEndTran <- function(channel, commit = TRUE)
+{
+    if(!odbcValidChannel(channel))
+         stop("first argument is not an open RODBC channel")
+    .Call(C_RODBCEndTran, attr(channel, "handle_ptr"), commit)
+}
+
+odbcDataSources <- function(type = c("all", "user", "system"))
+{
+    type <- match.arg(type)
+    type <- match(type, c("all", "user", "system"))
+    .Call(C_RODBCListDataSources, type)
+}
